// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer;

import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

public class Answer {

    private int points;
    private int status;
    private Date timestamp;
    // string to the file containing the answer. Used to fetch the answer for viewing
    private String file;

    public Answer(int p, Date d) {
	points = p;
	timestamp = d;
	status = 0;
    }

    public Answer(int p, int s, Date d) {
	this(p,d);
	status = s;
    }
    
    public Answer(int p, int s, Date d, String f) {
	this(p,s,d);
	file = f;
    }

    public void setFileName(String s) {
	file = s;
    }

    public String getFileName() {
	return file;
    }

    public void setPoints(int i) {
	points = i;
    }

    public int getPoints() {
	return points;
    }

    public void setStatus(int i) { status = i; }

    public int getStatus() { return status; }

    public void setTimestamp(Date d) {
  	timestamp = d;
    }

    public boolean isBadStatus() {
	return status != DataStorage.OK && status != DataStorage.OK_MODIFIED;
    }

    public boolean isFile() {
	return file != null;
    }

    public Date getTimeStamp() {
	return timestamp;
    }

    public String getFormattedTimestamp() {
	DateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
	return df.format(timestamp);
    }

}
