// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer;

import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;

import java.io.Serializable;

/**
 * This is an exercise, defined by an ExerciseTemplate, which has been
 * attached to a Round. It does not need to know what round it is attached to,
 * or what the actual exercise, defined by the correspondin ExerciseTemplate,
 * is.
 */

public class Exercise implements Serializable {
    private int /*points,*/ maxPoints, number, maxSubs;
    private ExerciseTemplate ex;
    // Additional Applet parameters defined here. Must contain only instances of AppletParameters
    private List additionalParameters;

    public Exercise(ExerciseTemplate e, int max, int nro) {
	//	points = 0;
	ex = e;
	if(max >= 0) {
	    maxPoints = max;
	} else {
	    maxPoints = 1;
	}
	number = nro;
	additionalParameters = null;
    }

    public Exercise(ExerciseTemplate e, int max, int nro, List params) {
	this(e,max,nro);
	additionalParameters = params;
    }

    public Exercise(ExerciseTemplate e, int max, int nro, int subs) {
	this(e,max,nro);
	maxSubs = subs;
    }

    public int getNumber() {
	return number;
    }

    public void setNumber(int value) {
	number = value;
    }

    public String getName() { return ex.getName(); }

    public int getMax() {
	return maxPoints;
    }

    public int getSubs() {
	return maxSubs;
    }

    public String getHintPage() { return ex.getHintPage(); }

    public String getDescription() { return ex.getDescription(); }

    /**
     * Sets the additional, round-specific applet parameters. To set 
     * exercise-specific parameters, use setAppletParameters() in ExerciseTemplate
     */
    public void setAdditionalAppletParameters(List l) {
	additionalParameters = l;
    }

    public List getAppletParameters() {
	List params = ex.getAppletParameters();
	// done this way to make it possible to modify exercise parameters in runtime.
	LinkedList ret = new LinkedList(params);
	if(additionalParameters != null)
	    ret.addAll(additionalParameters);
	return ret;
    }

    public ExerciseTemplate getExerciseTemplate() {
	return ex;
    }

    public void addUse() {
	ex.addUse();
    }

    public void removeUse() {
	ex.removeUse();
    }

    public void getUses() {
	ex.getUses();
    }

    public List getSupportedLocales() {
	return ex.getSupportedLocales();
    }


    public String getLocalizedName(String lang) { 
	return ex.getLocalizedName(lang);
    }

    public String getLocalizedDescription(String lang) {
	return ex.getLocalizedDescription(lang);
    }
    
    public String getInstructions() {
        return ex.getInstructions();
    }
    
    public String getLocalizedInstructions(String lang) {
        return ex.getLocalizedInstructions(lang);
    }    
    
    public String getTask() {
        return ex.getTask();
    }
    
    public String getLocalizedTask(String lang) {
        return ex.getLocalizedTask(lang);
    }
    
    public LinkedList getCodeBlockNames() {
        return ex.getCodeBlockNames();
    }
    
    public LinkedList getLocalizedCodeBlockNames(String lang) {
        return ex.getLocalizedCodeBlockNames(lang);
    }
    
    public String getCodeBlock(String blockName) {
        return ex.getCodeBlock(blockName);
    }
    
    public String getLocalizedCodeBlock(String blockName, String lang) {
        return ex.getLocalizedCodeBlock(blockName, lang);
    }
}
