// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.components;

import org.apache.tapestry.form.IPropertySelectionModel;
import org.apache.tapestry.html.BasePage;
import org.apache.tapestry.IRequestCycle;
import cexer.*;
import cexer.util.Logging;
import cexer.pages.MonitorViewStudentSubmissionPage;
import org.apache.tapestry.BaseComponent;
import java.util.List;
import java.util.Locale;

public abstract class ExerciseSubmissions extends BaseComponent {

    private Answer answer;
    

    public abstract MonitorSession getMonitorSession();

    public abstract String getRowStyle1();
    public abstract String getRowStyle2();

    public abstract void setIndex(int index);
    
    public abstract int getIndex();

    // Used when modifying points.
    // The hidden field uses this to get the index of the for loop.
    // (This is kind of a persistence hack.)
    public int getSubmissionIndex() {
        return getIndex();
    }
    
    // The hidden field uses this to report the index back to us when we push "Set"
    public void setSubmissionIndex(int index) {
        setAnswer((Answer)getAnswers().get(index));
    }

    
    public abstract int getEditingIndex();

    public abstract void setEditingIndex(int index);

    public abstract void setChangedPoints(int points);

    public abstract int getChangedPoints();

    public abstract void setWarning(boolean state);

    public int getPoints() {
        MonitorSession session = getMonitorSession();
        Course course = session.getCourse();
        return session.getMonitoredStudent().getResult(course, session.getRound(), getExercise());
    }

    public void setPoints(int points) {
        setChangedPoints(points);
    }

    public Exercise getExercise() {
        return getMonitorSession().getExercise();
    }

    public void setExercise(Exercise ex) {}

    public void setAnswers(List l) {}

    public List getAnswers() {
        MonitorSession session = getMonitorSession();
        DataStorage ds = DataStorage.getInstance();
        return ds.getAnswers(session.getCourse().getId(), 
                session.getRound().getId(), 
                session.getExercise().getNumber(), 
                session.getMonitoredStudent().getId());
    }

    public boolean isHasAnswers() {
        return getAnswers().size() > 0;
    }

    public void setAnswer(Answer a) {
        answer = a;
    }

    public Answer getAnswer() {
        return answer;
    }

    public String getStatusMessage() {
        String message = null;

        switch(answer.getStatus()) {
        case DataStorage.OK:
            message = "";
            break;
        case DataStorage.AFTER_DEADLINE:
            message = "Submission after deadline.";
            break;
        case DataStorage.INVALID_FINGERPRINT:
            message = "Invalid fingerprint.";
            break;
        case DataStorage.AFTER_CLOSING:
            message = "Submission after course has closed.";
            break;
        case DataStorage.ALL_SUBMISSIONS_USED:
            message = "All submissions used.";
            break;
        case DataStorage.OK_MODIFIED:
            message = "Status OK. Points modified.";
            break;
        case DataStorage.AFTER_DEADLINE_MODIFIED:
            message = "Submission after deadline. Points modified.";
            break;
        case DataStorage.INVALID_FINGERPRINT_MODIFIED:
            message = "Invalid fingerprint. Points modified";
            break;
        case DataStorage.AFTER_CLOSING_MODIFIED:
            message = "Submission after course has closed. Points modified.";
            break;
        case DataStorage.ALL_SUBMISSIONS_USED_MODIFIED:
            message = "All submissions used. Points modified.";
            break;
        }

        return message;
    }

    public void showSubmission(IRequestCycle cycle, int index) {
        MonitorViewStudentSubmissionPage next_page = (MonitorViewStudentSubmissionPage)cycle.getPage("MonitorViewStudentSubmission");
        next_page.setAnswer((Answer)getAnswers().get(index));
        setSubmissionIndex(index);
        cycle.activate(next_page);
    }

    public String getRowStyle() {
        if((getIndex() % 2)==1)
            return getRowStyle2();
        else return getRowStyle1();
    }

    public boolean isEditPoints() {
        if(getEditingIndex() == getIndex())
            return true;
        return false;
    }

    public void setEditPoints(IRequestCycle cycle) {
        Object[] params = cycle.getListenerParameters();
        setEditingIndex(((Integer)params[0]).intValue());
        cycle.activate(getPage());
        }

    public Round getRound() {
        return getMonitorSession().getRound();
    }
    
    public void modifyPointsClicked(int index) {
        setSubmissionIndex(index);
        Logging.info(this, "set submission index  = " + index);
    }

    public void changePoints() {
        DataStorage ds = DataStorage.getInstance();
        MonitorSession session = getMonitorSession();
        
        Answer answer = getAnswer();
        
        int newStatus = answer.getStatus();
        if(newStatus < 10) {
            newStatus += 10; //This marks the answer as "points changed from monitor ui"
        }
        
        boolean success = ds.updateResult(session.getMonitoredStudent().getId(),
                session.getCourse(),
                session.getRound(),
                session.getExercise(),
                getChangedPoints(),
                answer.getTimeStamp(),
                newStatus);
        if(success) {
            /*System.out.println("Success changing points! \nMonitor user " + session.getUserId() + 
                    " changed submission point for user " + 
                    session.getMonitoredStudent().getId() +
                    ", course '" + session.getCourse().getFullCourseName() +
                    ", exercise '" + session.getExercise().getName() +
                    "' new points: " + getChangedPoints()); */
            Logging.info(this, "Monitor user " + session.getUserId() + 
                    " changed submission point for user " + 
                    session.getMonitoredStudent().getId() +
                    ", course '" + session.getCourse().getFullCourseName() +
                    "', exercise '" + session.getExercise().getName() +
                    "', new points: " + getChangedPoints());

            Student s = session.getMonitoredStudent();
            s.editResult(session.getCourse(),session.getRound(),session.getExercise(),getChangedPoints());
            /*
            Results r = s.getResults(session.getCourse());
            int oldPoints = s.getResult(session.getCourse(), session.getRound(), session.getExercise());
            if(oldPoints > getChangedPoints())
                setChangedPoints(oldPoints);
            int subs = s.getSubmissions(session.getCourse(), session.getRound(), session.getExercise());
            r.setResults(session.getRound().getRoundNumber(), session.getExercise().getNumber(), getChangedPoints(), subs);
            */
            answer.setPoints(getChangedPoints());
            answer.setStatus(newStatus);
            setEditingIndex(-1);
        } else {
            setWarning(true);
        }
    }

    public PointSelectionModel getPointSelectionModel() {
        return new PointSelectionModel(getExercise().getMax());
    }

    private class PointSelectionModel implements IPropertySelectionModel {
        private int maxpoints;

        public PointSelectionModel(int maxpoints) {
            this.maxpoints = maxpoints;
        }

        public String getLabel(int index) {
            return Integer.toString(index) + " ";
        }

        public Object getOption(int index) {
            return new Integer(index);
        }

        public int getOptionCount() {
            return maxpoints + 1;
        }

        public String getValue(int index) {
            return Integer.toString(index);
        }

        public Object translateValue(String value) {
            return getOption(Integer.parseInt(value));
        }
    }
    
    public void detach() {
        answer = null;
    }
}