// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * 
 */
package cexer.components;

import org.apache.tapestry.form.PropertySelection;
import org.apache.tapestry.form.IPropertySelectionModel;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.Tapestry;

/**
 * @author cefallen
 *
 */
public abstract class RawOutputPropertySelection extends PropertySelection {

    public abstract boolean getRaw();
    
    protected void renderFormComponent(IMarkupWriter writer, IRequestCycle cycle)
    {
        renderDelegatePrefix(writer, cycle);

        writer.begin("select");
        writer.attribute("name", getName());

        if (isDisabled())
            writer.attribute("disabled", "disabled");

        if (getSubmitOnChange())
            writer.attribute("onchange", "javascript:   this.form.events.submit();");

        renderIdAttribute(writer, cycle);

        renderDelegateAttributes(writer, cycle);

        getValidatableFieldSupport().renderContributions(this, writer, cycle);
        
        // Apply informal attributes.
        renderInformalParameters(writer, cycle);

        writer.println();

        IPropertySelectionModel model = getModel();

        if (model == null)
            throw Tapestry.createRequiredParameterException(this, "model");

        int count = model.getOptionCount();
        boolean foundSelected = false;
        Object value = getValue();

        for (int i = 0; i < count; i++)
        {
            Object option = model.getOption(i);

            writer.begin("option");
            writer.attribute("value", model.getValue(i));

            if (!foundSelected && isEqual(option, value))
            {
                writer.attribute("selected", "selected");

                foundSelected = true;
            }

            //writer.print(model.getLabel(i));
            //This hack enables raw output:
            if (getRaw())
                writer.printRaw(model.getLabel(i));
            else
                writer.print(model.getLabel(i));
            
            writer.end();

            writer.println();
        }

        writer.end(); // <select>

        renderDelegateSuffix(writer, cycle);
    }
    
    private boolean isEqual(Object left, Object right)
    {
        // Both null, or same object, then are equal

        if (left == right)
            return true;

        // If one is null, the other isn't, then not equal.

        if (left == null || right == null)
            return false;

        // Both non-null; use standard comparison.

        return left.equals(right);
    }

}
