// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Locale;

import org.apache.tapestry.IBinding;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.RedirectException;
import org.apache.tapestry.engine.IEngineService;
import org.apache.tapestry.engine.ILink;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.html.BasePage;
import org.apache.tapestry.util.exception.ExceptionAnalyzer;
import org.apache.tapestry.util.exception.ExceptionDescription;

import cexer.MonitorSession;
import cexer.Student;
import cexer.StudentSession;
import cexer.util.MailerService;

public abstract class ExceptionPage extends BasePage implements PageRenderListener {
    
    private IBinding exceptionBinding;
    
    /* private Throwable exception; */

    public abstract StudentSession getStudentSession();
    
    public abstract String getErrorFeedback();
    
    public abstract IEngineService getRestartService();
    
    public abstract Throwable getException();
    
    public abstract void setException(Throwable exception);
    
    public abstract void setExceptionTrace(String stackTrace);
    
    public abstract String getExceptionTrace();
    
    public void pageBeginRender(PageEvent event) { 
        if(getException() != null)
            setExceptionTrace(getStackTrace(getException()));
    }
    
    public ILink restart(IRequestCycle cycle) {
        // do your own cleanup here        
        return getRestartService().getLink(false, null);
    }
    
    public String getErrorMessageText() {
        String message ="";
        String messageLevel = System.getProperty("exception.page.info.level");
        if(messageLevel == null)
            messageLevel = "";
        if(messageLevel.toLowerCase().equals("short") && getException() != null) {
            message += getException().getMessage();
        } else if(messageLevel.toLowerCase().equals("long") && getException() != null) {
            message += getStackTrace(getException());
        }
        return message;
    }
    
    public boolean getHasShortMessage() {
        String messageLevel = System.getProperty("exception.page.info.level");
        if(messageLevel == null)
            messageLevel = "";
        return messageLevel.toLowerCase().equals("short");
    }
    
    public boolean getHasLongMessage() {
        String messageLevel = System.getProperty("exception.page.info.level");
        if(messageLevel == null)
            messageLevel = "";
        return messageLevel.toLowerCase().equals("long");
    }
    
    public ExceptionDescription[] getExceptions() {
        ExceptionAnalyzer analyzer = new ExceptionAnalyzer();
        if(getException() == null)
            return new ExceptionDescription[] {new ExceptionDescription("","",null,null)}; 
        else return analyzer.analyze(getException());
        
    }
    
    /**
     * Asks the framework if the Visit-object exists, e.g. if the HttpSession has been created
     * or if the current session is stateless.
     * @return TRUE if the StudentSession-object is created, otherwise FALSE
     */
    public abstract boolean getStudentSessionState();
    
    public abstract MonitorSession getMonitorSession();
    
    public abstract boolean getMonitorSessionState();
    
    /**
     * @param cycle
     */
    public void formSubmit(IRequestCycle cycle) {
        MailerService ms = MailerService.getInstance();
        String message = "";
        if(getStudentSessionState() && getStudentSession() != null) {
            message += "The exception page was displayed for the following user:\n" +
                    "Student id: " + getStudentSession().getUserId();
            message += "\nCourse: " + getStudentSession().getCourse();
            message += "\nRound: " + getStudentSession().getRound();
            if(getStudentSession().getExercise() != null)
                message += "\nExercise id: " + getStudentSession().getExercise().getExerciseTemplate().getId();
        } else if(getMonitorSessionState() && getMonitorSession() != null) {
            message += "The exception page was displayed for the following monitor interface user:\n";
            message += "User id: " + getMonitorSession().getUserId();
            message += "\nCourse: " + getMonitorSession().getCourse();
        } else message = "The exception page was displayed for a sessionless user.";
        
        if(getErrorFeedback() != null && !getErrorFeedback().equals(""))
            message += "\n\nUser feedback (from Exception page): \n" + getErrorFeedback() + "\n";
        
        if(getExceptionTrace() != null)
            message += "\nThe following Exception was encountered: \n" + getExceptionTrace() + "\n\n"; 
        
        ms.mailToAdminList(this, message, null);
        throw new RedirectException(restart(cycle).getURL());
    }
    
    public void cancel(IRequestCycle cycle) {
        throw new RedirectException(restart(cycle).getURL());
    }
    
    /*
    public void setException(Throwable exception) throws Exception {
        this.exception = exception;
        throw new Exception(exception);
    }
    */
    
    public IBinding getExceptionBinding() {
        return exceptionBinding;
    }
    
    public void setExceptionBinding(IBinding exceptionBinding) {
        this.exceptionBinding = exceptionBinding;
    }
    
    public void detach() {
        super.detach();
        exceptionBinding = null;
        /* exception = null; */
    }
    
    private String getStackTrace(Throwable t)
    {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw, true);
        t.printStackTrace(pw);
        pw.flush();
        sw.flush();
        return sw.toString();
    }
}
