// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import java.util.*;

import org.apache.tapestry.IRequestCycle;
import cexer.*;
import cexer.util.*;

public abstract class ExercisePage extends SecurePage {

    /*
     * Tapesty 4 application state object. Fetches the injected student-session
     * object (see Login.page)
     */

    public abstract StudentSession getStudentSession();
    
    public Course getCourse() {
	return getStudentSession().getCourse();
    }

    public Round getRound() {
	return getStudentSession().getRound();
    }

    public Exercise getExercise() {
	return getStudentSession().getExercise();
    }

    public Student getStudent() {
	return getStudentSession().getStudent();
    }

    public String getName() {
	return getExercise().getLocalizedName(getLocale().getLanguage());
    }

    public String getDescription() {
	return getExercise().getLocalizedDescription(getLocale().getLanguage());
    }
    
    public boolean getHasCodeBlocks() {
        if(getExercise().getLocalizedCodeBlockNames(getLocale().getLanguage()).size() > 0) {
            //System.out.println("Exercise has " + getExercise().getLocalizedCodeBlockNames(getLocale().getLanguage()).size() + "code blocks");
            return true;
        }
        return false;
    }
    
    public String getTask() {
        return getExercise().getLocalizedTask(getLocale().getLanguage());
    }
    
    public String getInstructions() {
        return getExercise().getLocalizedInstructions(getLocale().getLanguage());
    }
    
    public LinkedList getCodeBlockNames() {
        return getExercise().getLocalizedCodeBlockNames(getLocale().getLanguage());        
    }
    
    public String getCodeBlock(String blockName) {
        return getExercise().getLocalizedCodeBlock(blockName, getLocale().getLanguage());          
    }

    /**
     * Tell if the exercise has a hint page
     * @return true if the exercise has a hint page
     */
    public boolean isHintPage() {
	String hintPage = getExercise().getHintPage();
	return !(hintPage == null || hintPage.equals(""));
    }

    public boolean isPrevious() {
	Round r = getRound();
	Exercise e = getExercise();
	List exercises = r.getExerciseList();
	int i = exercises.indexOf(e);
	if(i < 0) { 
	    return false;
	}
	if(i == 0) {
	    List rounds = getCourse().getRoundList();
	    if(rounds.indexOf(r) <= 0 || !((Round)rounds.get(rounds.indexOf(r)-1)).isOpen()) {
		return false;
	    }
	    return true;
	}
	return true;
    }

    public Object[] getPreviousData() {
	if(!isPrevious())
	    return null;
	Object[] params = new Object[2];
	Round r = getRound();
	Exercise e = getExercise();
	List exercises = r.getExerciseList();
	List rounds = getCourse().getRoundList();
	int ei = exercises.indexOf(e); // number of the exercise
	int ri = rounds.indexOf(r); // number of the round
	if(ei < 0) { // XXX this is an error
	    Logging.warn(this, "Index of exercise ("+e+") in round.getexerciseList() is negative (round "+r+")");
	    params[0] = new Integer(0);
	    params[1] = params[0];
	    return params;
	}
	if(ei == 0) {
	    ri = rounds.indexOf(r)-1;
	    r = (Round)rounds.get(ri);
	    exercises = r.getExerciseList();
	    ei = exercises.size()-1;
	} else {
	    ei--;
	}
	params[0] = new Integer(ri);
	params[1] = new Integer(ei);
	return params;
    }

    public boolean isNext() {
	Round r = getRound();
	Exercise e = getExercise();
	List exercises = r.getExerciseList();
	int i = exercises.indexOf(e);
	if(i < 0) { // XXX this is an error, should be handled
	    Logging.warn(this, "Index of exercise ("+e+") in round.getexerciseList() is negative (round "+r+")");
	    return false; 
	}
	if(i == exercises.size()-1) {
	    List rounds = getCourse().getRoundList();
	    if(rounds.indexOf(r) == rounds.size() -1) {
		return false;
	    } else if(!((Round)rounds.get(rounds.indexOf(r)+1)).isOpen()) {
		return false;
	    }
	    return true;
	}
	return true;
    }

    public Object[] getNextData() {
	if(!isNext())
	    return null;
	Object[] params = new Object[2];
	Round r = getRound();
	Exercise e = getExercise();
	List exercises = r.getExerciseList();
	List rounds = getCourse().getRoundList();
	int ei = exercises.indexOf(e);
	int ri = rounds.indexOf(r);
	if(ei < 0) { // XXX this is an error
	    Logging.warn(this, "Index of exercise ("+e+") in round.getexerciseList() is negative (round "+r+")");
	    params[0] = new Integer(0);
	    params[1] = params[0];
	    return params;
	}
	if(ei == exercises.size() -1) {
	    ri = rounds.indexOf(r)+1;
	    r = (Round)rounds.get(ri);
	    exercises = r.getExerciseList();
	    ei = 0;
	} else {
	    ei++;
	}
	params[0] = new Integer(ri);
	params[1] = new Integer(ei);
	return params;
    }

    public void changeExercise(IRequestCycle cycle) {
	StudentSession session = getStudentSession();
	Object[] params = cycle.getListenerParameters();
	int ri = ((Integer)params[0]).intValue();
	int ei = ((Integer)params[1]).intValue();
	List rounds = session.getCourse().getRoundList();
	Round r = (Round)rounds.get(ri);
	List exercises = r.getExerciseList();
	Exercise e = (Exercise)exercises.get(ei);
	session.setRound(r);
    session.setExercise(e);
	cycle.activate(this);
	return;
	    
    }


    public boolean isBeforeDeadLine() {
	boolean b = false;
	if(getStudent().hasPersonalDeadLine(getRound().getId())) {
	    b = getStudent().getDeadLine(getRound().getId()).after(Calendar.getInstance().getTime());
	}
	if(b == false)
	    return getRound().getDeadLine().after(Calendar.getInstance().getTime());
	return b;
    }

    public boolean getHasPersonalDeadline() {
	return getStudent().hasPersonalDeadLine(getRound().getId());
    }

    public String getPersonalDeadline() {
	return getStudent().getFormattedDeadLine(getRound().getId());
    }


//      public void main(IRequestCycle cycle) {
//  	cycle.activate("Main");
//      }

    public void detach() {
	super.detach();
    }

    /**
     * @return true if the exercise contains an applet
     */
    public boolean isExerciseHasApplet() {
	return !getExercise().getExerciseTemplate().getCode().equals("");
    }
    
    public String getFullCourseName() {
        return getCourse().getCourseCode() + " " + getCourse().getLocalizedName(getLocale().getLanguage());
    }
    
    public void linkToRound(IRequestCycle cycle) {
        MainPage nextPage = (MainPage)cycle.getPage("Main");
        nextPage.setAsOnlyMaximizedRound(getStudentSession().getRound());
        cycle.activate(nextPage);
    }
    
    public abstract int getTabIndex();
    public abstract int getTabTextIndex();
    public abstract String getCodeBlockName();
    
    public String getCodeBlockTabClass() {
        if(getTabIndex() == 0)
            return "current";
        return "";
    }
    
    public String getCodeBlockTabAction() {
        return "showTab(this, '" + getCodeBlockName() +  "'); return false;";
    }
    
    public String getCodeBlockTextStyle() {
        if(getTabTextIndex() != 0)
            return "display: none";
        else return "";
    }
    
    public String getCodeBlockId() {
        return getCodeBlockName().toLowerCase() + "-text";
    }
    
    public boolean getHasNewDescription() {
        if(!getTask().equals("")) {
            //System.out.println("Task = " + getTask());
            return true;
        }
        return false;
    }
    
    public boolean getHasInstructions() {
        if(!getInstructions().equals("")) {
            return true;
        }
        return false;
    }
}
