// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;


import org.apache.tapestry.html.BasePage;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.IAsset;
import org.apache.tapestry.Tapestry;

import cexer.*;
import cexer.util.chart.ChartProvider;
import cexer.util.chart.ChartAsset;
import cexer.util.CourseSelectionModel;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import java.util.Iterator;
import java.util.GregorianCalendar;
import java.util.Calendar;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.text.NumberFormat;
import java.awt.Paint;
import java.awt.Color;

import org.jCharts.Chart;
import org.jCharts.chartData.DataSeries;
import org.jCharts.chartData.AxisChartDataSet;
import org.jCharts.properties.AreaChartProperties;
import org.jCharts.properties.LineChartProperties;
import org.jCharts.properties.ChartProperties;
import org.jCharts.properties.AxisProperties;
import org.jCharts.properties.LegendProperties;
import org.jCharts.types.ChartType;
import org.jCharts.axisChart.AxisChart;
import org.jCharts.chartData.ChartDataException;
import org.jCharts.properties.AxisTypeProperties;
import org.jCharts.properties.StackedBarChartProperties;
import org.jCharts.properties.ClusteredBarChartProperties;
import org.jCharts.properties.BarChartProperties;
import org.jCharts.properties.PointChartProperties;

import java.awt.Stroke;
import java.awt.Shape;

import org.jCharts.test.TestDataGenerator;

import org.apache.tapestry.form.*;

public abstract class MonitorCompareCoursesPage extends SecurePage implements ChartProvider {
    
    private CourseWrapper current;
    private List courses;

    private IAsset points;
    private IAsset grades;
    private boolean showResults;
    private NumberFormat nf;

    private double[][] pointData;
    private double[][] gradeData;

    public final static Integer TRUE = new Integer(1);
    public final static Integer FALSE = new Integer(0);
    public final static String POINTS = "pointcomparison";
    public final static String GRADES = "gradecomparison";
    public static Paint[] paints = new Paint[]{ Color.BLUE, Color.RED, Color.LIGHT_GRAY, Color.BLACK, Color.GREEN, Color.ORANGE, Color.DARK_GRAY, Color.YELLOW, Color.PINK, Color.CYAN, Color.MAGENTA };

    public abstract MonitorSession getMonitorSession();
    
    public abstract Integer getAbsolutePoints();

    public abstract void setAbsolutePoints(Integer i);

    public abstract Integer getAbsoluteStudents();

    public abstract void setAbsoluteStudents(Integer i);

    public abstract Integer getGradeStyle();

    public abstract void setGradeStyle(Integer i);

    public abstract Integer getAccumulatingPoints();

    public abstract void setAccumulatingPoints(Integer i);

    public abstract boolean isComparePoints();

    public abstract void setComparePoints(boolean b);

    public abstract boolean isGraphics();

    public abstract void setGraphics(boolean b);

    public abstract boolean isText();

    public abstract void setText(boolean b);

    public abstract boolean isCompareGrades();

    public abstract void setCompareGrades(boolean b);   
    
    public abstract Course getCourse();
    
    public IPropertySelectionModel getModel() {
        /*List cl = getCourseList();
        Iterator li = cl.iterator();
        String[] courses = new String[cl.size()];
        for(int i=0;li.hasNext();i++) {
            courses[i] = ((CourseWrapper)li.next()).getCourse().getCourseName();
        }
        StringPropertySelectionModel sm = new StringPropertySelectionModel(courses);
        return sm;
        */
        return new CourseSelectionModel(getMonitorSession().getStudent().getCourseList());
    }
    
    public void setModel(StringPropertySelectionModel model) {}
    
   
    private NumberFormat getNumberFormat() {
	if(nf == null) {
	    nf = NumberFormat.getInstance(getLocale());
	    nf.setMinimumFractionDigits(0);
	    nf.setMaximumFractionDigits(3);
	}
	return nf;
    }

    public void compareCourses(IRequestCycle cycle) {
	setShowResults(true);
	cycle.activate(this);
    }
    
    public void selectCourses(IRequestCycle cycle) {
        setShowResults(false);
        cycle.activate(this);
    }
    
    // returns a List containing the CourseWrappers for each course
    public List getCourseList() {
	/*if(courses == null) {
	    List l = getMonitorSession().getStudent().getCourseList();
	    courses = new LinkedList();
	    Iterator i = l.iterator();
	    while(i.hasNext()) {
		Course c = (Course)i.next();
		courses.add(new CourseWrapper(c));
	    }
	}*/
	//return courses;
    return getSelectedList();
    }

    public abstract List getSelectedList();
    public abstract void setSelectedList(List l); 
    
    public void setCourseList(List l) {
        setSelectedList(l);
	courses = l;
    }

    public void setCurrentCourse(CourseWrapper c) {
	current = c;
    }

    public CourseWrapper getCurrentCourse() {
	return current;
    }

    // returns the localized name of the current course
    public String getLocalizedName() {
	return getCourse().getCourseCode() + " " + getCourse().getLocalizedName(getLocale().getLanguage());
    }

    public void detach() {
	super.detach();
	showResults = false;
	pointData = null;
	gradeData = null;
	//courses = null;
    }

    public IAsset getPointComparisonImage() {
	if(points == null) {
	    points = new ChartAsset(getRequestCycle(),this,POINTS);
	}
	return points;
    }

    public IAsset getGradeComparisonImage() {
	if(grades == null) {
	    grades = new ChartAsset(getRequestCycle(),this,GRADES);
	}
	return grades;
    }

    public String getComparisonText() {
	List compared = getComparedCourses();
	String ret = "";
	Iterator iter;
	int i = 1;
	int count = 0;
	ret = ret + "<table><tr>";
	double[][] pdata = getPointComparisonData(compared, getPointComparisonDataLength(compared));
	double[][] gdata = null;
	int length = 0;
	String[] grades = null;
	if(isCompareGrades()) {
	    iter = compared.iterator();
	    // for long form grading, find the course with most grades
	    if(TRUE.intValue() == getGradeStyle().intValue()) {
		while(iter.hasNext()) {
		    Course c = (Course)iter.next();
		    grades = c.getGrades();
		    if(grades.length > length)
			length = grades.length;
		}
	    } 
	    gdata = getGradeComparisonData(compared, grades, length);
	}
	iter = compared.iterator();
	
	while(iter.hasNext()) {
	    ret = ret + "<td>";
	    Course c = (Course)iter.next();
	    ret = ret + getComparison(c, pdata, gdata, count++);
	    ret = ret + "</td>";
	    if(i % 3 == 0) {
		ret = ret + "</tr><tr>";
	    }
	    i++;
	}
	ret = ret + "</tr></table>";
	return ret;
    }

    private String getComparison(Course c, double[][] pdata, double[][]gdata, int count) {
	DataStorage ds = DataStorage.getInstance();
	Statistics stat = ds.getStatisticsForCourse(c.getId());
	String ret = "<p>";
    /*
    String pref = "";
    String postf = "</tr>";
    if(count==0) {
        ret = "<table>" + ret;
        pref = "<tr>";
        postf = "";
    }
    */
	ret = ret + "<strong>Course Name:</strong> " + c.getLocalizedName(getLocale().getLanguage()) + "<br/>";
	ret = ret + "<strong>Number of students:</strong> " + stat.getNroStudents() + "<br/>";
	ret = ret + "<strong>Number of Submissions:</strong> " + stat.getNroAnswers() + "<br/>";
	if(isComparePoints()) {
	    ret = ret + "<h3>Point distribution";
	    if(FALSE.equals(getAbsolutePoints())) {
		ret = ret + "(persentage of points on course)";
	    }
	    ret = ret + "</h3>";
	    //ret = ret + "</td>" + postf;
	    for(int i = 0; i < pdata[count].length;i++) {
		if(TRUE.equals(getAbsolutePoints())) {
		    ret = ret + i + " points: " + getNumberFormat().format(pdata[count][i]);
		} else if(FALSE.equals(getAbsolutePoints())) {
		    ret = ret + i + " % of max: " + getNumberFormat().format(pdata[count][i]);
		}
		if(FALSE.equals(getAbsoluteStudents())) {
		    ret = ret + "%";
		}
		ret = ret + "<br/>";
	    }
	    ret = ret + "<br/><hr/>";
	}
	if(isCompareGrades()) {
	    ret = ret + "<h3>Grade distribution";
	    if(FALSE.equals(getGradeStyle())) {
		ret = ret + "(failed, passed, max grade, max points)";
	    }
	    ret = ret + "</h3>";
	    ret = ret + "<p>";
	    if(FALSE.equals(getGradeStyle())) {
		ret = ret + "<strong>Max points:</strong> " + getNumberFormat().format(gdata[3][count]);
		if(FALSE.equals(getAbsoluteStudents())) {
		    ret = ret + "%";
		}
		ret = ret + "<br/>";
		ret = ret + "<strong>Max grade:</strong> " + getNumberFormat().format(gdata[2][count]);
		if(FALSE.equals(getAbsoluteStudents())) {
		    ret = ret + "%";
		}
		ret = ret + "<br/>";
		ret = ret + "<strong>Passed:</strong> " + getNumberFormat().format(gdata[1][count]);
		if(FALSE.equals(getAbsoluteStudents())) {
		    ret = ret + "%";
		}
		ret = ret + "<br/>";
		ret = ret + "<strong>Failed:</strong> " + getNumberFormat().format(gdata[0][count]);
		if(FALSE.equals(getAbsoluteStudents())) {
		    ret = ret + "%";
		}
		ret = ret + "<br/>";
	    } else {
		String[] grades = c.getGrades();
		ret = ret + "<strong> grade 0:</strong> " + getNumberFormat().format(gdata[count][0]);
		if(FALSE.equals(getAbsoluteStudents())) {
		    ret = ret + "%";
		} 
		ret = ret + "<br/>";
		for(int i = 1; i < gdata[count].length; i++) {
		    ret = ret + "<strong> grade " + grades[i-1] + ":</strong> " + getNumberFormat().format(gdata[count][i]);
		    if(FALSE.equals(getAbsoluteStudents())) {
			ret = ret + "%";
		    }
		    ret = ret + "<br/>";
		}
	    }
	    if(count==0)
            ret = ret + "<br/>";
         ret = ret + "<br/>";
	}
	return ret;
    }

    private List getComparedCourses() {
	/*
     List compared = new LinkedList();
	List wrapped = getCourseList();
	Iterator i = wrapped.iterator();
	while(i.hasNext()) {
	    CourseWrapper cw = (CourseWrapper)i.next();
	    if(cw.isSelected()) {
		compared.add(cw.getCourse());
	    }
	}
	return compared;
    */
        return getSelectedList();
    }

    public Chart getChart(String chartName) 
	throws ChartDataException {
	//System.out.println("Starting chart generation for chart " + chartName);
	List compared = getComparedCourses();
	if(POINTS.equals(chartName)) {
	    return getPointChart(compared);
	} 
	if(GRADES.equals(chartName)) {
	    Iterator i = compared.iterator();
	    while(i.hasNext()) {
		Course c = (Course)i.next();
		if(c.getGradeMinPoints() == null || c.getGradeMinPoints().length == 0) {
		    i.remove();
		}
	    }
	    return getGradeChart(compared);
	}
	//System.out.println("No chart");
	return null;
    }

    public Chart getChart() throws ChartDataException {
	return getChart(POINTS);
    }

    private double[][] getPointComparisonData(List compared, int length) {
	// data creation;
	if(pointData == null) {
	    pointData = new double[compared.size()][length+1];
	    Iterator iter = compared.iterator();
	    DataStorage ds = DataStorage.getInstance();
	    int count = 0;
	    while(iter.hasNext()) {
		Course c = (Course)iter.next();
		List students = ds.getCourseResults(c);
		Iterator siter = students.iterator();
		while(siter.hasNext()) {
		    Student s = (Student)siter.next();
		    int result = s.getCourseResult(c);
		    if(FALSE.intValue() == getAbsolutePoints().intValue()) {
			result = (int)((double)100*result/c.getMax());
		    }
		    pointData[count][result]++;
		}
		if(FALSE.intValue() == getAbsoluteStudents().intValue()) {
		    double nroStudents = students.size();
		    double modifier = ((double)100)/nroStudents;
		    for(int i = 0; i < pointData[count].length; i++) {
			pointData[count][i] = pointData[count][i]*modifier;
		    }
		}
		if(TRUE.equals(getAccumulatingPoints())) {
		    double follow = 0;
		    for(int i = 0; i < pointData[count].length; i++) {
			pointData[count][i] = pointData[count][i] + follow;
			follow = pointData[count][i];
		    }
		}
		count++;
	    }

	}
	return pointData;
    }

    private int getPointComparisonDataLength(List compared) {
	int length = 0;
	if(TRUE.intValue() == getAbsolutePoints().intValue()) {
	    Iterator iter = compared.iterator();
	    while(iter.hasNext()) {
		Course c = (Course)iter.next();
		int max = c.getMax();
		if(max > length)
		    length = max;
	    }
	} else {
	    length = 100;
	}
	return length;
    }

    private Chart getPointChart(List compared) throws ChartDataException {
	if(compared.size() == 0) {
	    //System.out.println("No courses selected");
	    return null;
	}
	String xtitle = "points";
	String ytitle = "Number of Students";
	String title = "Comparison of points";
	Course c;
	
	int length = getPointComparisonDataLength(compared);

	double[][] data = getPointComparisonData(compared, length);

	String[] labels = new String[length+1];
	for(int i = 0; i < labels.length;i++) {
	    labels[i] = ""+i;
	}
	DataSeries series = new DataSeries(labels,xtitle,ytitle,title);

	String[] labelTitles = new String[compared.size()];
	for(int i = 0; i < labelTitles.length; i++) {
	    c = (Course)compared.get(i);
	    labelTitles[i] = c.getCourseCode() + " " + c.getLocalizedName(getLocale().getLanguage());
	}
	Paint[] p = new Paint[labelTitles.length];
	for(int i = 0; i < p.length; i++) {
	    p[i] = paints[i%paints.length];
	}
	Stroke[] strokes= new Stroke[labelTitles.length];
	Shape[] shapes= new Shape[labelTitles.length];
	for(int i = 0; i < strokes.length;i++) {
	    strokes[i] = LineChartProperties.DEFAULT_LINE_STROKE;
	    shapes[i] = PointChartProperties.SHAPE_CIRCLE;
	}
	LineChartProperties barChartProperties = new LineChartProperties(strokes, shapes);
	AxisChartDataSet axisData = new AxisChartDataSet(data, labelTitles,p,ChartType.LINE,barChartProperties);
	series.addIAxisPlotDataSet(axisData);
	return createActualChart(series, 800, 600, false);
    }


    private double[][] getGradeComparisonData(List compared, String[] grades, int length) {
	if(gradeData == null) {
	    if(TRUE.intValue() == getGradeStyle().intValue()) {
		gradeData = new double[compared.size()][length+1];
	    } else {
		gradeData = new double[4][compared.size()];
	    }
	    Iterator iter = compared.iterator();
	    DataStorage ds = DataStorage.getInstance();
	    int count = 0;
	    while(iter.hasNext()) {
		Course c = (Course)iter.next();
		List students = ds.getCourseResults(c);
		Iterator siter = students.iterator();
		if(TRUE.intValue() == getGradeStyle().intValue()) {	    
		    grades = c.getGrades();
		    while(siter.hasNext()) {
			Student s = (Student)siter.next();
			String grade = s.getGrade(c);
			if(grade.equals("0")) {
			    gradeData[count][0]++;
			} else {
			    for(int j = 0; j < grades.length; j++) {
				if(grade.equals(grades[j])) {
				    gradeData[count][j+1]++;
				    break;
				}
			    }
			}
		    }
		} else {
		    int maxPoints = c.getMax();
		    int min = c.getMin();
		    int[] minPoints = c.getGradeMinPoints();
		    int maxGrade = 0;
		    if(minPoints.length > 0) {
			maxGrade = minPoints[minPoints.length-1];
		    }
		    while(siter.hasNext()) {
			Student s = (Student)siter.next();
			int points = s.getCourseResult(c);
			if(points < min) {
			    gradeData[0][count]++;
			} else if(points < maxGrade) {
			    gradeData[1][count]++;
			} else if(points < maxPoints) {
			    gradeData[2][count]++;
			} else {
			    gradeData[3][count]++;
			}
		    }
		}
		if(FALSE.intValue() == getAbsoluteStudents().intValue()) {
		    double nroStudents = students.size();
		    double modifier = ((double)100)/nroStudents;
		    if(TRUE.intValue() == getGradeStyle().intValue()) {	    		
			for(int i = 0; i < gradeData[count].length; i++) {
			    gradeData[count][i] = gradeData[count][i]*modifier;
			}
		    } else {
			for(int i = 0; i < gradeData.length; i++) {
			    gradeData[i][count] = gradeData[i][count]*modifier;
			}
		    }
		}
		count++;
	    }
	}
	return gradeData;
    }

    private Chart getGradeChart(List compared) throws ChartDataException {
	//System.out.println("generating grade chart");
	if(compared.size() == 0) {
	    //System.out.println("No courses selected");
	    return null;
	}
	String xtitle = "grades";
	String ytitle = "Number of Students";
	if(FALSE.intValue() == getGradeStyle().intValue()) {	
	    xtitle = "courses";
	    ytitle = "grades";
	}
	String title = "Comparison of distribution of grades";
	Course c;
	int length = 0;
	Iterator iter = compared.iterator();
	String[] grades = null;
	// for long form grading, find the course with most grades
	if(TRUE.intValue() == getGradeStyle().intValue()) {
	    while(iter.hasNext()) {
		c = (Course)iter.next();
		grades = c.getGrades();
		if(grades.length > length)
		    length = grades.length;
	    }
	} 
	String[] labels;
	// in long formthe x-axis contains grades, in short form it contains the courses
	if(TRUE.intValue() == getGradeStyle().intValue()) {
	    labels = new String[length+1];
	} else {
	    labels = new String[compared.size()];
	}
	if(TRUE.intValue() == getGradeStyle().intValue()) {
	    for(int i = 0; i < labels.length;i++) {
		labels[i] = ""+i;
	    }
	} else {
	    for(int i = 0; i < compared.size();i++) {
		c = (Course)compared.get(i);
		labels[i] = c.getCourseCode() + " " + c.getLocalizedName(getLocale().getLanguage());
	    }
	}
	DataSeries series = new DataSeries(labels,xtitle,ytitle,title);

	// data creation;
	double[][] data = getGradeComparisonData(compared, grades, length);
	String[] labelTitles;
	if(TRUE.intValue() == getGradeStyle().intValue()) {
	    labelTitles = new String[compared.size()];
	    for(int i = 0; i < labelTitles.length; i++) {
		c = (Course)compared.get(i);
		labelTitles[i] = c.getCourseCode() + " " + c.getLocalizedName(getLocale().getLanguage());
	    }
	} else {
	    labelTitles = new String[4];
	    labelTitles[0] = "not passed";
	    labelTitles[1] = "passed";
	    labelTitles[2] = "max grade";
	    labelTitles[3] = "max points";
	}
	//Paint[] paints = TestDataGenerator.getRandomPaints(labelTitles.length);
	Paint[] p = new Paint[labelTitles.length];
	for(int i = 0; i < p.length; i++) {
	    p[i] = paints[i%paints.length];
	}
	if(TRUE.intValue() == getGradeStyle().intValue()) {
	    ClusteredBarChartProperties barChartProperties = new ClusteredBarChartProperties();
	    AxisChartDataSet axisData = new AxisChartDataSet(data, labelTitles,p,ChartType.BAR_CLUSTERED,barChartProperties);
	    series.addIAxisPlotDataSet(axisData);
	} else {
	    StackedBarChartProperties barChartProperties = new StackedBarChartProperties();
	    AxisChartDataSet axisData = new AxisChartDataSet(data, labelTitles,p,ChartType.BAR_STACKED,barChartProperties);
	    series.addIAxisPlotDataSet(axisData);
	}
	return createActualChart(series, 800, 600, false);
    }

    private Chart createActualChart(DataSeries series, int width, int height, boolean horizontal) {
        ChartProperties chartProperties = new ChartProperties();
        chartProperties.setEdgePadding(20f);

        // FIXME: this does not compile -TA
        //AxisProperties axisProperties = new AxisProperties(horizontal);
        //axisProperties.getYAxisProperties().setShowGridLines(AxisTypeProperties.GRID_LINES_ALL);
        //axisProperties.getYAxisProperties().setShowTicks(AxisTypeProperties.TICKS_ALL);
        AxisProperties axisProperties = new AxisProperties();

        LegendProperties legendProperties = new LegendProperties();
        legendProperties.setNumColumns(2);

        Chart result = new AxisChart( series, chartProperties, axisProperties, legendProperties, width, height );
        return result;
    }

    public boolean isShowResults() {
        return showResults;
    }
    
    
    public void setShowResults(boolean b) {
	showResults = b;
    // v3: Tapestry.fireObservedChange(this, "showResults", b);
	// Tapestry.fireObservedChange(this, "showResults",new Boolean(b)); some weird error.. commented out
    }
    
    public MonitorChartPopupPage chartPopup(IRequestCycle cycle, String chartName) {
        IAsset image = null;
        if(chartName.equals("grades"))
            image = getGradeComparisonImage();
        if(chartName.equals("points"))
            image = getPointComparisonImage();
        MonitorChartPopupPage nextPage = (MonitorChartPopupPage)cycle.getPage("MonitorChartPopup");
        nextPage.setImage(image);
        nextPage.setName(chartName);
        return nextPage;
    }

}
