// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import java.util.List;

import org.apache.tapestry.html.BasePage;
import cexer.*;

public class MonitorCourseStatisticsPage extends SecurePage {

    private Course course;
    private Statistics stat;
    private RoundStatistics rstat;
    private ExerciseStatistics estat;
    private int pointIndex;
    private int times;
    // accumulating nro of students
    private int accumul;

    public Course getCourse() {
	return course;
    }

    public void setPointAmount(Integer i) {
	if(i != null) {
	    times = i.intValue();
	    accumul += times;
	}
    }

    public Integer getPointAmount() {
	// hack to include those students that returned nothing
	if(getPointIndex() == 0) {
	    int nr = getAmountOfStudents() - getRound().getAmountOfStudents();
	    times += nr;
	    accumul += nr;
	}
  	return new Integer(times);
    }

    public void setAccumulatingAmount(int i) {}

    public int getAccumulatingAmount() {
	return accumul;
    }

    public void setAccumulatingPersentage(int i) {}

    public int getAccumulatingPersentage() {
	return (int) (100*((float)accumul / (float)getAmountOfStudents()));	
    }

    public void setPersentage(int i) {}

    public int getPersentage() {
	return (int) (100*((float)times / (float)getAmountOfStudents()));
    }

    public void setCourse(Course c) {
	course = c;
    }

    public void setRound(RoundStatistics rs) {
	rstat = rs;
	accumul = 0;
	//System.out.println("Accumul is now " + accumul);
    }

    public RoundStatistics getRound() {
	return rstat;
    }

    public void setExercise(ExerciseStatistics es) {
	estat = es;
    }
    
    public ExerciseStatistics getExercise() {
	return estat;
    }

    public int getAmountOfStudents() {
	getStatistics();
	return stat.getNroStudents();
    }

    public int getAverageFromMaximum() {
	return (int) (100*((float)getExercise().getAverageOfMaximums()) / ((float)getExercise().getMax()));
    }

    public int getAverageFromAll() {
	return (int) (100*
		      (((float)getExercise().getAverageOfMaximums()) / ((float)getExercise().getMax()))
		       * (((float)getExercise().getAmountOfStudents()) / ((float)getAmountOfStudents())));
    }

    public int getPersentageOfStudents() {
	float a = (float)getExercise().getAmountOfStudents();
	float b = getAmountOfStudents();
	return (int) (100*(a/b));
    }

    public Statistics getStatistics() {
	if(stat == null) {
	    Visit visit = (Visit)getVisit();
	    DataStorage ds = DataStorage.getInstance();
	    stat = ds.getStatisticsForCourse(visit.getCourse().getId());
	}
	return stat;
    }
    
    public List getRoundStatistics() {
	getStatistics();
	return stat.getRounds();
    }

    public int getNumberOfAnswers() {
	getStatistics();
	return stat.getNroAnswers();
    }

    public void setPointIndex(int i) {
	pointIndex = i;
    }

    public int getPointIndex() {
	return pointIndex;
    }

    public void detach() {
	super.detach();
	course = null;
	stat = null;
	rstat = null; 
	estat = null;
    }

}
