// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.pages;

import java.util.Locale;

import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.valid.ValidationDelegate;
import cexer.*;
import cexer.util.Checker;
import cexer.util.Logging;
import cexer.util.LocaleSelectionModel;
import cexer.util.PasswordService;

public abstract class SettingsPage extends SecurePage implements PageRenderListener {

    /*
     * Tapesty 4 application state object. Fetches the injected student-session
     * object (see Login.page)
     */
    public abstract StudentSession getStudentSession();
        
    public abstract void setWarning(boolean state);    
    
    public abstract void setWarningText(String warningText);
    
    public abstract String getStudentNumber();
    
    public abstract void setStudentNumber(String studentNumber);
    
    public abstract String getNewPassword();
    
    public abstract String getNewPassword2();
    
    public abstract void setNewPassword(String value);

    public abstract void setNewPassword2(String value);
    
    public abstract void setCurrPassword(String value);

    public abstract String getCurrPassword();
    
    public abstract String getName();
    
    public abstract void setName(String name);
    
    public abstract Course getCourse();
        
    public abstract String getEmail();
    
    public abstract void setEmail(String email);
    
    public abstract Locale getLanguage();
    
    public abstract void setLanguage(Locale lang);
    
    public void pageBeginRender(PageEvent event) { 
        Student student = getStudentSession().getStudent();
        setName(student.getName());
        setStudentNumber(student.getStudentNumber());
        setEmail(student.getEmail());
        setLanguage(new Locale(student.getLanguage()));
    }
    
    public void formSubmit(IRequestCycle cycle) {
        StudentSession session = getStudentSession();
        Student s = session.getStudent();
        
        setWarning(false);
        ValidationDelegate delegate = (ValidationDelegate) getBeans().getBean("delegate");

        boolean changingPwd = (getNewPassword() != null && !getNewPassword().trim().equals("")) || (getNewPassword2() != null && !getNewPassword2().trim().equals(""));
        if (changingPwd) { // first check that the passwords are ok
            if (getCurrPassword() == null) {
                setWarning(true);
                setWarningText("You must give current password to change password");
                cycle.activate(this);
                return;
            }
            if (!s.matchPassword(getCurrPassword())) {
                setWarning(true);
                setWarningText("Current password does not match");
                cycle.activate(this);
                return;
            }
            if (getNewPassword() != null && !(getNewPassword().equals(""))
                    && !getNewPassword().equals(getNewPassword2())) {
                setWarning(true);
                setWarningText("New passwords do not match");
                cycle.activate(this);
                return;
            }
            if (getNewPassword() != null && getNewPassword().length() > 12) {
                setWarning(true);
                setWarningText("Password too long (max 12 characters).");
                cycle.activate(this);
                return;
            }
        }
        if (delegate.getHasErrors()) {
            // clearFields();
            cycle.activate(this);
            return;
        }

        DataStorage ds = DataStorage.getInstance();
        String studentId = s.getId();
        if (getName() != null && !getName().equals(s.getName())) {
            ds.setStudentField(studentId, "name", getName());
            s.setName(getName());
        }

        if (changingPwd) {
            PasswordService passwordService = PasswordService.getInstance();
            ds.setStudentField(studentId, "password", passwordService.encrypt(getNewPassword()));
            s.setPassword(passwordService.encrypt(getNewPassword()));
        }

        if (getEmail() != null && !getEmail().equals(s.getEmail())) {
            ds.setStudentField(studentId, "email", getEmail());
            s.setEmail(getEmail());
        }

        Logging.info(this, "getStudentNumber(): " + getStudentNumber());
        

        if (getStudentNumber() != null
                && getStudentNumber().length() > 0
                && getStudentNumber().length() < 13
                && !getStudentNumber().equals(s.getStudentNumber())) {
            ds.setStudentField(studentId, "studentNumber", getStudentNumber());
            s.setStudentNumber(getStudentNumber());
        }

        if (getLanguage() != null && !getLanguage().getLanguage().equals(s.getLanguage())) {
            ds.setStudentField(studentId, "lang", getLanguage().getLanguage());
            s.setLanguage(getLanguage().getLanguage());
            cycle.getEngine().setLocale(getLanguage());
            cycle.setListenerParameters(new String[] { getPage().getPageName(), "app" });
            setWarning(true);
            setWarningText("Changes saved.");
            cycle.activate("LanguageChange");
            return;
        }

        clearFields();
        setWarning(true);
        setWarningText("Changes saved.");
    }

    public boolean isChangePasswordAllowed() {
        StudentSession session = getStudentSession();

        if (session.getAuthMethod() == StudentSession.PASSWORD) {
            return true;
        } else {
            return false;
        }
    }

    public boolean isChangeStudentNumberAllowed() {
        StudentSession session = getStudentSession();

        if (session.getAuthMethod() == StudentSession.SHIBBOLETH) {
            return true;
        } else {
            return false;
        }
    }

    private void clearFields() {
        setCurrPassword("");
        setNewPassword("");
        setNewPassword2("");
    }

    public LocaleSelectionModel getLocaleModel() {
        LocaleSelectionModel lsm = DataStorage.getInstance().getLocaleSelectionModel();
        return lsm;
    }

}
