// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.util;

/**
 * Cache containing student numbers for cexer.
 * Uses the numeric starting part of a student number
 * as the key.
 */

import java.util.regex.Pattern;
import cexer.Student;

public class StudentCache extends ArrayCache {

    Pattern p;

    public StudentCache() {
        super();
        p = Pattern.compile("[(\\D)*]");
    }

    public StudentCache(int size) {
        super(size);
        p = Pattern.compile("[(\\D)*]");
    }

    public Student get(String key) {
        //int k = getIntPart(key);
        // next method call is the synchronized part
        Student s = (Student)get(key);

        if (s != null && s.getId().equals(key)) {
            return s;
        }

        return null;
    }

    public void put(Student s, String key) {
        //int k = getIntPart(key);
        // next method call is the synchronized part
        insert(s,key);
    }

    private int getIntPart(String s) {
        String[] pat = p.split(s);
        int k = 0;

        try {
            if(s.charAt(0) == 'T' || s.charAt(0) == 'S') {
                k = Integer.parseInt(pat[1]);
            } else {
                k = Integer.parseInt(pat[0]);
            }
        } catch(NumberFormatException e) {
            Logging.info(this, "Error in studentnumber "+s, e);
            e.printStackTrace();
        } catch(ArrayIndexOutOfBoundsException ex) {
            Logging.info(this, "Error in studentnumber "+s, ex);
            ex.printStackTrace();
        }
        return k;
    }

}
