// Copyright 2007 Tapio Auvinen, Erik Fallenius, Ville Karavirta,
// Jussi Nikander, and Harri Salonen
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package cexer.util.chart;

import javax.servlet.ServletException;

// import org.apache.tapestry.engine.AbstractService;
// import org.apache.tapestry.engine.IEngineServiceView;
// import org.apache.tapestry.request.ResponseOutputStream;

import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

import org.apache.hivemind.util.Defense;
import org.apache.tapestry.IComponent;
import org.apache.tapestry.IPage;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.engine.IEngineService;
import org.apache.tapestry.engine.ILink;
import org.apache.tapestry.error.RequestExceptionReporter;
import org.apache.tapestry.services.LinkFactory;
import org.apache.tapestry.services.ServiceConstants;
import org.apache.tapestry.util.ContentType;
import org.apache.tapestry.web.WebResponse;

         
import org.jCharts.Chart;
import org.jCharts.encoders.JPEGEncoder;
import org.jCharts.chartData.ChartDataException;
import org.jCharts.properties.PropertyException;


import cexer.util.Logging;
import org.apache.tapestry.util.io.DataSqueezerImpl;
import org.apache.tapestry.util.io.StringAdaptor;

/** ChartService.java
 * Taken from Tapestry 4.0.1 Workbench application with only minor changes in exception handling. (Using cexer.util.Logging) 
 * 
 * @author cefallen
 *
 */

//v3: public class ChartService extends AbstractService {
public class ChartService implements IEngineService {

    public static final String SERVICE_NAME = "chart";  

    /** @since 4.0 */
    private RequestExceptionReporter _exceptionReporter;

    /** @since 4.0 */
    private LinkFactory _linkFactory;

    /** @since 4.0 */
    private WebResponse _response;
    
    /* v3:
    public ILink getLink(IRequestCycle cycle, IComponent component, Object[] params) {  

	String[] context;
	String pageName = component.getPage().getPageName();
	String idPath = component.getIdPath();
	context = new String[2];
	context[0] = pageName;
	context[1] = idPath;
	return constructLink(cycle, SERVICE_NAME, context, params, true);
    }
    */  

    public ILink getLink(boolean post, Object parameter)
    {
        Defense.isAssignable(parameter, ChartAsset.class, "parameter");

        ChartAsset chartAsset = (ChartAsset) parameter;
        IComponent component = chartAsset.getProvider();

        Map parameters = new HashMap();

        parameters.put(ServiceConstants.SERVICE, getName());
        parameters.put(ServiceConstants.PAGE, component.getPage().getPageName());
        parameters.put(ServiceConstants.COMPONENT, component.getIdPath());
        parameters.put(ServiceConstants.PARAMETER, new String[] { chartAsset.getChartName() });
        
        return _linkFactory.constructLink(this, false, parameters, true);
    }
 

   
    /* v3:
    public void service(IEngineServiceView view, IRequestCycle cycle, ResponseOutputStream output) 
    throws ServletException, IOException {

	//System.out.println("Starting ChartService");

	Object[] params = getParameters(cycle);
	String[] context = getServiceContext(cycle.getRequestContext());
	
	String pageName = context[0];
	String idPath = context[1];
	String chartName = null;
	//System.out.println("First try block starts");
	try {
	    if(params != null) {
		chartName = (String) params[0];
	    }
	}catch(ClassCastException e) {
	    chartName = null;
	    Logging.warn(this, "ChartService could not fetch chartName for page " + pageName + " since " + params[0] + 
			 " was not a String");
	    //System.out.println("Poks1");
	}
	//System.out.println("First try block ends with idPath " + idPath);
	IPage page = cycle.getPage(pageName);
	IComponent component = null;
	if(idPath == null || idPath.equals("null")) 
	    component = page;
	else
	    component = page.getNestedComponent(idPath);
	//System.out.println("Starting image fetch");
	try {
	    ChartProvider provider = (ChartProvider) component;
	    //System.out.println("Fetching image");
	    Chart chart = provider.getChart(chartName);
	    if(chart == null) {
		//System.out.println("Bummer, got null image!");
	    }
	    //System.out.println("Got image");
	    output.setContentType("image/jpeg");
	    // apparently this may cause some instability which can be fixed with 
	    // synchronization. Should be tested if it this holds true for TRAKLA2.
	    synchronized(this) {
		JPEGEncoder.encode(chart,1.0f,output);
	    }
	    //System.out.println("ChartService done");
	    return;
	} catch(ClassCastException e) {
	    Logging.warn(this, "IComponent " + component + " did not implement ChartProvider interface, could not create chart");
	    //System.out.println("Poks2");
	    e.printStackTrace();
	    return;
	}catch(ChartDataException e) {
	    Logging.warn(this, "Problem in chart data, could not create chart. " + e.getMessage());
	    e.printStackTrace();
	    return;
	}catch(PropertyException e) {
	    Logging.warn(this, "Incorrect chart properties, could not create chart. " + e.getMessage());
	    //System.out.println("Poks4");
	    e.printStackTrace();
	    return;
	} catch(Exception e) {
	    e.printStackTrace();
	    return;
	}
    }   
    */
    
    public void service(IRequestCycle cycle) throws IOException
    {
        String pageName = cycle.getParameter(ServiceConstants.PAGE);
        String componentId = cycle.getParameter(ServiceConstants.COMPONENT);
                
        IPage page = cycle.getPage(pageName);
        IComponent component = page.getNestedComponent(componentId);
        
        String chartName = null;
        
        /* Create datasqueezer for extracting/decoding chartname from cycle.getParameters. */
        DataSqueezerImpl dsq = new DataSqueezerImpl();
        dsq.register(new StringAdaptor()); // Register decoder for strings (chartname in string format)
        
        Object[] param = dsq.unsqueeze(cycle.getParameters(ServiceConstants.PARAMETER));
        
        if(param != null && !((String)param[0]).equals("")) {
            chartName = (String)param[0];
        }else {
            Logging.warn(this, "ChartService could not fetch chartName for page " + pageName + " since " + param[0] + " was not a String");
        }
        try
        {
            ChartProvider provider = (ChartProvider) component;

            Chart chart = provider.getChart(chartName);

            OutputStream output = _response.getOutputStream(new ContentType("image/jpeg"));

            // I've seen a few bits of wierdness (including a JVM crash) inside this code.
            // Hopefully, its a multi-threading problem that can be resolved
            // by synchronizing.

            synchronized (this)
            {
                JPEGEncoder.encode(chart, 1.0f, output);
            }
            return;
        }
        catch (ClassCastException ex)
        {
            Logging.warn(this, "IComponent " + component + " did not implement ChartProvider interface, could not create chart.");
            ex.printStackTrace();
            return;
        }catch(ChartDataException ex) {
            Logging.warn(this, "Problem in chart data, could not create chart. " + ex.getMessage());
            ex.printStackTrace();
            return;
        } catch(Exception ex) {
            ex.printStackTrace();
            return;
        }
        //return;
    }

    public String getName()
    {
        return SERVICE_NAME;
    }
    
    /** @since 4.0 */
    public void setExceptionReporter(RequestExceptionReporter exceptionReporter)
    {
        _exceptionReporter = exceptionReporter;
    }

    /** @since 4.0 */
    public void setLinkFactory(LinkFactory linkFactory)
    {
        _linkFactory = linkFactory;
    }

    /** @since 4.0 */
    public void setResponse(WebResponse response)
    {
        _response = response;
    }
    
}
