/* $Id: tables.sql,v 1.16 2007/09/06 13:22:39 vkaravir Exp $ */
/*
   Table to store the user info
*/
CREATE TABLE Users (
    id VARCHAR(255) PRIMARY KEY,
    password VARCHAR(32),
    randomSeed BIGINT,
    email VARCHAR(255) default '',
    name VARCHAR(255) default '',
    lang VARCHAR(3),
    defcourse integer default 0,
    studentNumber VARCHAR(12)
);


/*
   Table to store course data
*/
CREATE TABLE course (
    code VARCHAR(20) NOT NULL,
    id INT CHECK (id >= 0) PRIMARY KEY,
    minPoints DECIMAL CHECK (minPoints >= 0) DEFAULT 0,
    open TIMESTAMP without time zone,
    close TIMESTAMP without time zone,
    organization_id INT,
    organizer_uid VARCHAR(20)
);

/*
   Table to store the localized course names
*/
CREATE TABLE coursedescription (
    cid INT REFERENCES Course(id),
    name VARCHAR(255),
    lang VARCHAR(3),
    description TEXT,
    link VARCHAR(1024)
);


CREATE TABLE CourseGrading (
    cId INT REFERENCES Course(id),
    grade varchar(20),
    minPoints INT CHECK (minPoints >= 0)
);


CREATE TABLE organization (
    id INT CHECK (id >= 0) PRIMARY KEY,
    shiburl text
);


CREATE TABLE organizationdescription (
    orgid INT NOT NULL REFERENCES Organization(id),
    lang VARCHAR(3) NOT NULL,
    acronym VARCHAR(16),
    name VARCHAR(255),
    link VARCHAR(1024),
    description TEXT
);



CREATE TABLE Round (
    cId INT REFERENCES Course(id),
    id INT PRIMARY KEY,
    deadline TIMESTAMP WITHOUT TIME ZONE NOT NULL,
    min DECIMAL CHECK (min >= 0) DEFAULT 0,
    number INT NOT NULL,
    makeupFactor DECIMAL CHECK (0 <= makeupFactor AND makeupFactor <= 1),
    open TIMESTAMP WITHOUT TIME ZONE NOT NULL
);

/*
   Table to store rounds
*/
CREATE TABLE RoundDescription (
    rid INT REFERENCES Round(id),
    name VARCHAR(255),
    lang VARCHAR(3),
    description TEXT
);

/*
   Table to store exercise templates
*/
CREATE TABLE exercise (
       id INT CHECK (id >= 0) PRIMARY KEY,
       type smallint DEFAULT -1 NOT NULL
);

CREATE TABLE appletexercise (
       code VARCHAR(255) NOT NULL,
       height SMALLINT CHECK (height > 0) NOT NULL,
       width SMALLINT CHECK (width > 0) NOT NULL,
       hintPage VARCHAR(255),
       codebase VARCHAR(255),
       archive VARCHAR(255),
       eid INT REFERENCES Exercise(id)
);

/*
   Table to store the descriptions of the exercise templates. For localization
   purposes. The lang attribute should be a valid ISO language Code. See 
   http://www.ics.uci.edu/pub/ietf/http/related/iso639.txt.
*/
CREATE TABLE exercisedescription (
       description TEXT NOT NULL,
       name VARCHAR(255) NOT NULL,
       lang VARCHAR(3) NOT NULL,
       eId INT REFERENCES Exercise(id)
);

/*
   Table to store the applet parameters

   Type points to the type of the class where the parameter is specified,
   so either Course (c), Exercise (e) or Round (r).
   Id should somehow be specified as a "foreign key" to point to the id
   of the type. Problem is, the id's aren't all the same type. Plus don't
   know how one key could point to many tables.
*/
CREATE TABLE appletparameter (
       value VARCHAR(255) NOT NULL,
       name VARCHAR(255) NOT NULL,
       type char(1) CHECK (type = 'c' OR type = 'e' OR type = 'r'),
       id INT
);


CREATE TABLE results (
    points SMALLINT CHECK (points >= 0) NOT NULL,
    tStamp TIMESTAMP WITHOUT TIME ZONE NOT NULL,
    userId VARCHAR(12) REFERENCES users(id),
    eId INT REFERENCES Exercise(id),
    cId INT REFERENCES Course(id),
    rId INT REFERENCES Round(id),
    status INT default 0,
    filename text
);

CREATE TABLE registrations (
    userid VARCHAR(255) REFERENCES users(id),
    cid INT REFERENCES course(id),
    userpriviledge INT default 0
);

CREATE TABLE Exercises (
       rId INT REFERENCES Round(id),
       eId INT REFERENCES Exercise(id),
       number INT NOT NULL,
       maxPoints SMALLINT CHECK (maxPoints >= 0),
       maxSubs SMALLINT CHECK (maxSubs >= 0) DEFAULT 32767,
       PRIMARY KEY (rId, number)
);


CREATE TABLE PersonalDeadLine (
    userid  VARCHAR(255) REFERENCES users(id) NOT NULL,
    rid INT REFERENCES round(id) NOT NULL,
    deadLine TIMESTAMP WITHOUT TIME ZONE NOT NULL,
    grantedby VARCHAR(255) REFERENCES users(id) NOT NULL
);

CREATE TABLE parameter (
       value VARCHAR(255) NOT NULL,
       name VARCHAR(255) NOT NULL,
       type char(1) CHECK (type = 'c' OR type = 'e' OR type = 'r'),
       id INT
);

CREATE TABLE Link (
    name VARCHAR(255) NOT NULL,
    target VARCHAR(255) NOT NULL
);

CREATE TABLE usershibinfo (
    id character varying(255) NOT NULL,
    studentnumber character varying(12),
    commonname character varying(256),
    displayname character varying(128),
    givenname character varying(128),
    surname character varying(128),
    email character varying(320),
    gender character(1) DEFAULT 'u'::bpchar,
    organization character varying(128),
    organizationname character varying(255),
    lang character varying(3),
    CONSTRAINT usershibinfo_gender CHECK ((((gender = 'm'::bpchar) OR (gender = 'f'::bpchar)) OR (gender = 'u'::bpchar)))
);

CREATE TABLE recoverpassword (
  id varchar(255) NOT NULL,
  hash varchar(255) NOT NULL
);
